<?php

if ( !defined( 'FW' ) ) {
    die( 'Forbidden' );
}

class FW_Extension_Update_Checker extends FW_Extension {

    /**
     * Plugin slug which is registered to the update server
     *
     * @var        string
     */
    static $SLUG = null;

    /**
     * Plugin absolute path. Change this to point to your plugin
     *
     * @var        string
     */
    static $PLUGIN_ABS_FILE = null;

    /**
     * WP Options Table key, where token would be stored
     *
     * @var        string
     */
    static $TOKEN_OPTIONS = null;

    /**
     * Server URL. Change this to point to your server
     *
     * @var        string
     */
    const SERVER = 'http://crumina.net';

    /**
     * Activation endpoint. Do not change this
     *
     * @var        string
     */
    const ACTIVATION_ENDPOINT = 'wpupdate/v1/activate';

    /**
     * Api key form input name.
     *
     * @var        string
     */
    const CODE_OPTIONS = 'envato_activate_code';

    /**
     * Api key revoke form input name.
     *
     * @var        string
     */
    const CODE_OPTIONS_REVOKE = 'envato_activate_code_revoke';

    /**
     * Cookie name for redirekt param.
     *
     * @var        string
     */
    const REDIRECTED_COOKIE = 'redirected_to_activate';

    /**
     * Option page slug
     *
     * @var        string
     */
    const PAGE_SLUG = 'activate_theme';

    /**
     * Update Server Variable
     *
     * @var        Puc_v4_Factory
     */
    protected $update_server;

    protected function _init() {
        self::$SLUG            = get_template();
        self::$TOKEN_OPTIONS   = self::$SLUG . '_theme_update_token';
        self::$PLUGIN_ABS_FILE = get_template_directory() . '/functions.php';

        require_once $this->locate_path( '/plugin/plugin-update-checker.php' );

        // Add admin activation page
        add_action( 'admin_menu', array( $this, 'adminMenu' ) );

        // Register activation form fields and redirect to activate page if needed
        add_action( 'admin_init', array( $this, 'adminInit' ) );

        $this->checkForUpdates();
    }

    public function adminMenu() {
        add_submenu_page( 'themes.php', __( 'Activate Theme', 'fw' ), __( 'Activate Theme', 'fw' ), 'administrator', self::PAGE_SLUG, array( $this, 'renderPageTemplate' ) );
    }

    public function checkForUpdates() {
        if ( !current_user_can( 'update_themes' ) ) {
            return false;
        }

        $status_transient = get_site_transient( self::CODE_OPTIONS );
        $status           = $status_transient ? $status_transient : $this->currentActivationStatus( $this->getCode() );

        if ( $this->getToken() && $status[ 'activated' ] ) {
            $this->update_server = Puc_v4_Factory::buildUpdateChecker( $this->getUpdateServerUrl(), self::$PLUGIN_ABS_FILE, self::$SLUG );
        } else {
            add_action( 'admin_notices', array( $this, 'activationNotice' ) );
        }
    }

    /**
     * Show activation notice if token is not found
     */
    public function activationNotice() {
        echo '<div class="notice notice-error"><p>' . __( 'Theme is not activated. To get automatic updates and other features, please activate Theme.', 'fw' );
        echo ' <a href="' . add_query_arg( array(
            'page' => 'activate_theme',
        ), admin_url( 'admin.php' ) ) . '" class="button button-primary">' . __( 'Activate Theme', 'fw' ) . '</a>';
        echo '</p></div>';
    }

    /**
     * Gets the update server url.
     *
     * @return     string  The update server url.
     */
    protected function getUpdateServerUrl() {
        $token_data = $this->getToken();
        $domain     = $this->getDomain();
        $query_args = array(
            'wpq_wp_update_token'  => $token_data,
            'wpq_wp_update_domain' => $domain,
            'wpq_wp_update_action' => 'get_metadata',
            'wpq_wp_update_slug'   => self::$SLUG,
        );
        return add_query_arg( $query_args, self::SERVER );
    }

    /**
     * Get current activation status with flag and message.
     *
     * Use this to show activation status within your plugin.
     *
     * @param      string  $purchase_code  The purchase code
     *
     * @return     array   associative array with 'activated' flag (bool) and 'msg'
     */
    public function currentActivationStatus( $purchase_code ) {
        $token_data = $this->getToken();
        if ( $this->getActivationStatus( $purchase_code ) ) {
            $return = array(
                'activated' => true,
                'msg'       => __( 'Your Theme is activated and receiving automatic updates.', 'fw' ),
                'color'     => 'green'
            );
        } elseif ( '' != $purchase_code && '' == $token_data ) {
            $return = array(
                'activated' => false,
                'msg'       => __( 'The purchase code is invalid. Please enter a correct one.', 'fw' ),
                'color'     => 'red'
            );
        } elseif ( '' == $token_data ) {
            $return = array(
                'activated' => false,
                'msg'       => __( 'Your Theme is not activated. Please enter your purchase code.', 'fw' ),
                'color'     => 'red'
            );
        } else {
            $return = array(
                'activated' => false,
                'msg'       => __( 'Your Theme is activated for another domain. Activating it here will remove activation from the other domain.', 'fw' ),
                'color'     => 'red'
            );
        }

        set_site_transient( self::CODE_OPTIONS, $return, 3600 );

        return $return;
    }

    /**
     * Sets the token from purchase code.
     *
     * @param      string   $purchase_code  The purchase code
     *
     * @return     boolean  whether successfully stored or not. On success, returns the json output from the server
     */
    public function setTokenFromCode( $purchase_code ) {
        // Delete activate status cache
        delete_site_transient( self::CODE_OPTIONS );

        if ( '' == $purchase_code ) {
            // Reset activation
            update_option( self::$TOKEN_OPTIONS, '' );
            return false;
        }
        $result = wp_remote_post( $this->getActivationUrl(), array(
            'body' => array(
                'purchase_code' => $purchase_code,
                'domain'        => $this->getDomain(),
            ), update_server
        ) );
        if ( is_wp_error( $result ) ) {
            return false;
        }

        $json = json_decode( $result[ 'body' ], true );

        if ( isset( $json[ 'token' ] ) && !empty( $json[ 'token' ] ) ) {
            update_option( self::$TOKEN_OPTIONS, $json[ 'token' ] );
        } else {
            update_option( self::$TOKEN_OPTIONS, '' );
        }

        return $json;
    }

    /**
     * Gets the activation status.
     *
     * @param      string   $purchase_code  The purchase code
     *
     * @return     boolean  The activation status.
     */
    public function getActivationStatus( $purchase_code ) {
        $result = wp_remote_get( $this->getActivationUrl(), array(
            'body' => array(
                'purchase_code' => $purchase_code,
                'domain'        => $this->getDomain(),
            ),
        ) );
        if ( is_wp_error( $result ) || 200 != $result[ 'response' ][ 'code' ] ) {
            return false;
        }

        $json = json_decode( $result[ 'body' ], true );

        if ( $this->getDomain() == $json[ 'domain' ] && $this->getToken() == $json[ 'token' ] ) {
            return true;
        } else {
            return false;
        }
    }

    /**
     * Gets the activation url.
     *
     * @return     string  The activation url.
     */
    protected function getActivationUrl() {
        return self::SERVER . '/wp-json/' . self::ACTIVATION_ENDPOINT . '/' . self::$SLUG;
    }

    /**
     * Gets the domain of this WordPress installation
     *
     * @return     string  The domain.
     */
    protected function getDomain() {
        return $_SERVER[ 'HTTP_HOST' ];
    }

    /**
     * Gets the token from WP Options Table
     *
     * @return     string  The token.
     */
    protected function getToken() {
        return get_option( self::$TOKEN_OPTIONS, '' );
    }

    /**
     * Run functions in admin init
     */
    public function adminInit() {
        $this->registerFormFields();
        $this->redirectToActivate();
    }

    /**
     * Redirect to activate page if needed
     */
    public function redirectToActivate() {
        $redirected = filter_input( INPUT_COOKIE, self::REDIRECTED_COOKIE );

        if ( !$redirected && !$this->getToken() ) {
            setcookie( self::REDIRECTED_COOKIE, 1, time() + 604800 );

            wp_safe_redirect( add_query_arg( array(
                'page' => self::PAGE_SLUG,
            ), admin_url( 'admin.php' ) ) );
            exit;
        }
    }

    /**
     * Register activation form fields
     */
    public function registerFormFields() {
        register_setting( self::CODE_OPTIONS, self::CODE_OPTIONS, array( $this, 'saveActivateCode' ) );
        add_settings_field( self::CODE_OPTIONS, __( 'Envato purchase code', 'fw' ), function() {
            
        }, self::PAGE_SLUG );

        register_setting( self::CODE_OPTIONS_REVOKE, self::CODE_OPTIONS_REVOKE, array( $this, 'revokeActivateCode' ) );
        add_settings_field( self::CODE_OPTIONS_REVOKE, __( 'Envato purchase code revoke', 'fw' ), function() {
            
        }, self::PAGE_SLUG );
    }

    /**
     * Revoke activation code
     */
    public function revokeActivateCode( $revoke ) {
        if ( $revoke ) {
            // Delete activate status cache
            delete_site_transient( self::CODE_OPTIONS );

            update_option( self::$TOKEN_OPTIONS, '' );
            update_option( self::CODE_OPTIONS, '' );
        }

        return $revoke;
    }

    /**
     * Set activate code
     */
    public function saveActivateCode( $code ) {
        if ( $code ) {
            $this->setTokenFromCode( wp_unslash( $code ) );
        }

        return $code;
    }

    /**
     * Get api code
     */
    public function getCode() {
        return get_option( self::CODE_OPTIONS, '' );
    }

    /**
     * Render option page template
     */
    public function renderPageTemplate() {
        $code                = $this->getCode();
        $status              = $this->currentActivationStatus( $code );
        $logo                = $this->get_config( 'logo-url' );
        $logo_item           = $this->get_config( 'logo-item-url' );
        $img_path            = $this->locate_URI( '/static/img' );
        $view_path           = $this->locate_path( '/views/activate-page.php' );
        $code_options_revoke = self::CODE_OPTIONS_REVOKE;
        $code_options        = self::CODE_OPTIONS;
        $theme               = wp_get_theme();

        wp_enqueue_style( 'activate-theme-page', $this->locate_URI( '/static/css/activate-theme-page.css' ), false, $this->manifest->get_version() );
        echo fw_render_view( $view_path, compact( 'img_path', 'code', 'status', 'logo', 'logo_item', 'code_options_revoke', 'code_options', 'theme' ) );
    }

}
